/*
 * fgr.h
 *
 * @author: Feiyi Wang <fwang2@ornl.gov>
 *
 */

#ifndef FGR_H_
#define FGR_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdbool.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <inttypes.h>
#include <limits.h>

#define NID_MAX     (19200)
#define RTR_MAX     (432)
#define OST_MAX     (56)
#define LNET_BASE   (201)
#define GNI_BASE    (101)
#define LNET_MAX    (36)
#define CLIENT_MAX  (36 * 56)
#define OST_PER_FS  (1008)
#define LNET_HALF   (18)
#define OSS_NUM_MAX (244)
#define OST_NUM_MAX (2016)
#define OST_PER_OSS (7)

typedef enum {ATLAS1, ATLAS2, ATLAS} fs_t;

/**
 * Initialize fgr placement, must call before any other fgr routines
 *
 * @param path pointing to configuration file: x.map, y.map and z.map
 *     alternatively, you can supply NULL, but export PIO_CONFIG_PATH
 *
 * @return you must check return value and bail out if return false
 */
bool fgr_init(const char *path);

/**
 * Convert a list of nids to corresponding osts. Note that we didn't turn on
 * a switch to randomize the starting point, thus the allocation is determistic
 * at this point. However, this should not be taken as the default case in the future.
 *
 * @nids  expect an array of _unique_ node ids. If your input is a list MPI rank ids
 *        we won't check its validity and will still allocate osts, but the results
 *        will be skewed.
 *
 * @osts  a list of matching osts index to be used for corresponding nid. It is the
 *        responsibility of calling routine to allocate approprate buffer, and thus its
 *        responsibility of free this buffer as well.
 *
 * @len   the number of nids
 *
 * @fs    ATLAS1, ATLAS2, or ATLAS. For ATLAS, you need the return range is
 *        0 - 2015. You need to take that into consideration.
 *
 * @return true for success, false for failure
 *
 */
bool fgr_nid2ost(uint32_t* nids, uint32_t * osts, size_t len, fs_t fs);

/**
 * Free all resources allocated for placement. You must call this at the end,
 * otherwise, memory leak will happen.
 */
void fgr_finalize(void);


/* debugging */
void fgr_lnet_debug(void);
void fgr_rtr_debug(void);
void fgr_ost_debug(void);
void fgr_oss_debug(void);

#ifdef __cplusplus
} /* extern "C" */
#endif

#endif /* FGR_H_ */
